#!/usr/bin/env python3

import os

import crash_test.error_codes
from crash_test.error_logger import log_error


def instance_name_check(instance_name) -> bool:
    """
    Check if the provided instance name is valid
    :param instance_name: the name of the multipass instance
    :return: Bool: False if the provided instance name is not valid;
                   True if the provided instance name is valid.
    """
    if "_" in instance_name:
        correct_format: str = instance_name.replace("_", "-")
        print(f"{log_error(error_code=crash_test.error_codes.INVALID_INSTANCE_NAME_ERROR)}{correct_format}.")
        return False
    else:
        return True


def project_check(project_path) -> bool:
    """
    Check if the provided project is in the pwd and exists
    :param project_path: project name
    :return: Bool: False if the provided project is a single file.
                   True if the provided project exists.
    """
    if os.path.isfile(project_path):
        # if project_path is a file will return False
        print(log_error(error_code=crash_test.error_codes.SINGLE_FILE_ERROR))
        return False

    if os.path.exists(project_path):
        return True
    else:
        print(log_error(error_code=crash_test.error_codes.NO_SUCH_FILE_OR_DIRECTORY_ERROR, project_path=project_path))
        return False


def arguments_check(instance_name, project_path) -> bool:
    """
    Check the instance name format and project
    :return: bool: True if the checks pass;
                   False if the checks fails.
    """
    return instance_name_check(instance_name) and project_check(project_path)



#!/usr/bin/env python3

import os

import crash_test.error_codes
from crash_test.error_logger import log_error
from crash_test.script_generator import script_generator


def check_dependencies(project_path: str) -> str:
    if os.path.exists(project_path):
        project_name = project_path[project_path.rfind("/") + 1:]
        for element in os.listdir(project_path):
            match element:
                case "requirements.txt":
                    return script_generator(project_name=project_name, project_type="python")
                case _:
                    print(log_error(error_code=crash_test.error_codes.NO_SUPPORTED_REQUIREMENTS_FILE_FOUND_ERROR))
    else:
        print(log_error(error_code=crash_test.error_codes.NO_SUCH_FILE_OR_DIRECTORY_ERROR, project_path=project_path))



#!/usr/bin/env python3

from typing import Final


def script_generator(project_name: str, project_type: str) -> str:
    script_content = ""
    commands: Final[dict] = {
        "python": [
            ['#!/usr/bin/env bash\n'],
            ['echo "Executing: sudo apt-get update"'],
            ['sudo apt-get update\n'],
            ['echo "Executing: sudo apt-get upgrade -y"'],
            ['sudo apt-get upgrade -y\n'],
            ['echo "Executing: sudo apt-get install python3 python3-pip python3-venv -y"'],
            ['sudo apt-get install python3 python3-pip python3-venv -y\n'],
            ['echo "Creating the venv..."'],
            [f'python3 -m venv ./{project_name}/venv\n'],
            ['echo "Activating the venv..."'],
            [f'source ./{project_name}/venv/bin/activate\n'],
            ['echo "Installing requirements..."'],
            [f'pip3 install -r ./{project_name}/requirements.txt']
        ]
    }
    for command in commands.get(project_type):
        script_content_join = "".join(command)
        script_content = script_content + script_content_join + "\n"

    return script_content



#!/usr/bin/env python3

import argparse
import subprocess
from shutil import which
from sys import exit
from typing import List

from colorama import Fore, Style

import crash_test.error_codes
from crash_test._version import __version__
from crash_test.args_checker import arguments_check
from crash_test.dependecies_checker import check_dependencies
from crash_test.error_logger import log_error


def args_parser():
    parser = argparse.ArgumentParser(
        description="Create a multipass instance to test your dAnGeRoUs project",
        formatter_class=argparse.RawDescriptionHelpFormatter
    )

    parser.add_argument("-i",
                        "--instance-name",
                        type=str,
                        required=True,
                        help="Multipass instance name"
                        )
    parser.add_argument("-p",
                        "--project",
                        type=str,
                        required=True,
                        help="Project folder to transfer to the multipass instance to be tested"
                        )
    parser.add_argument("-d",
                        "--delete",
                        action="store_true",
                        help="Delete the instance"
                        )
    parser.add_argument("--install-dependencies",
                        action="store_true",
                        help="Installs the dependencies for the project"
                        )
    parser.add_argument("-v",
                        "--version",
                        action="version",
                        version=f"%(prog)s {__version__}")

    return parser.parse_args()


class CrashTest:

    def __init__(self, args):
        self.args = args

        if self.args is not None and self.args.project.endswith("/"):
            self.args.project = self.args.project[:-1]

            # The project name without the path
            self.project_name = self.args.project[self.args.project.rfind("/") + 1:]

    def run(self):
        self.create_instance()

    @staticmethod
    def execute_multipass_command(command) -> None:
        """
        Executes a specified multipass command and prints possible errors
        :param command: the Multipass command to execute
        """
        result = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)

        if result.stdout:
            print(f"Output: {result.stdout}")

        if result.returncode != 0:
            print(
                f"{log_error(error_code=crash_test.error_codes.MULTIPASS_GENERIC_ERROR)}{result.stderr}{Style.RESET_ALL}"
            )
            exit(result.returncode)

    def install_dependencies(self) -> None:
        """
        Create the installation script for the dependencies in the multipass instance and executes it.
        """
        script_content = check_dependencies(self.args.project)

        if script_content:
            print(f"{Fore.GREEN}Installing dependencies...{Style.RESET_ALL}")

            create_script_command = ["multipass", "exec", f"{self.args.instance_name}", "--", "sh", "-c",
                                     f'echo "{script_content}" > ./{self.project_name}/python_dependencies.sh']

            self.execute_multipass_command(create_script_command)

            run_script_command = ["multipass", "exec", f"{self.args.instance_name}", "--", "bash",
                                  f"./{self.project_name}/python_dependencies.sh"]

            self.execute_multipass_command(run_script_command)
        else:
            print(log_error(error_code=crash_test.error_codes.NO_DEPENDENCIES_FOUND_ERROR))

    def create_instance(self) -> None:
        """
        Creates a Multipass instance and transfer the specified project to the newly created instance
        """
        if which("multipass") is not None:
            if arguments_check(instance_name=self.args.instance_name, project_path=self.args.project):
                # creates multipass session
                print(f"{Fore.GREEN}Creating multipass instance...\n{Style.RESET_ALL}")
                multipass_launch_command: List[str] = ["multipass", "launch", "--name", self.args.instance_name]
                self.execute_multipass_command(multipass_launch_command)
                print(f"{Fore.GREEN}Instance {self.args.instance_name} created successfully!\n{Style.RESET_ALL}")

                # transfers the specified project to the multipass session
                print(f"{Fore.GREEN}Transferring the project...\n{Style.RESET_ALL}")
                multipass_transfer_command: List[str] = ["multipass", "transfer", "-r", f"{self.args.project}/",
                                                         f"{self.args.instance_name}:."]
                self.execute_multipass_command(multipass_transfer_command)
                print(f"{Fore.GREEN}{self.project_name} transferred successfully!\n{Style.RESET_ALL}")

                # Install the dependencies
                if self.args.install_dependencies:
                    self.install_dependencies()

                # Opens a shell to the multipass instance
                print(f"{Fore.GREEN}Opening the shell...\n{Style.RESET_ALL}")
                multipass_shell_command: List[str] = ["multipass", "shell", self.args.instance_name]
                subprocess.run(multipass_shell_command)

                # Delete the instance if the --delete flag is specified
                if self.args.delete:
                    self.delete_instance()
        else:
            print(log_error(error_code=crash_test.error_codes.MULTIPASS_NOT_INSTALLED_ERROR))

    def delete_instance(self) -> None:
        """
        Delete the multipass instance if the -d --delete flag is specified
        """
        match input(
            "Are you sure you want to delete this instance? [Y/n]\n>>> "
        ).lower().strip()[0]:
            case "y":
                # Stops the instance
                print(f"\n{Fore.GREEN}Stopping the instance...{Style.RESET_ALL}")
                multipass_stop_command: List[str] = ["multipass", "stop", self.args.instance_name]
                self.execute_multipass_command(multipass_stop_command)
                print(f"{Fore.GREEN}Instance {self.args.instance_name} stopped.{Style.RESET_ALL}\n")

                # Deletes the instance
                print(f"{Fore.GREEN}Deleting the instance {self.args.instance_name}...{Style.RESET_ALL}\n")
                multipass_delete_command = ["multipass", "delete", self.args.instance_name]
                self.execute_multipass_command(multipass_delete_command)
                print(f"{Fore.GREEN}Instance deleted!{Style.RESET_ALL}")
            case _:
                print(f"{Fore.YELLOW}Elimination aborted.{Style.RESET_ALL}")


def main():
    crashtest: CrashTest = CrashTest(args=args_parser())
    crashtest.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

from colorama import Fore, Style

import crash_test.error_codes


def log_error(error_code: int, project_path: str = "") -> str:
    match error_code:
        case crash_test.error_codes.NO_SUCH_FILE_OR_DIRECTORY_ERROR:
            return f"crashtest: error: cannot access {project_path}: No such file or directory."
        case crash_test.error_codes.SINGLE_FILE_ERROR:
            return "Please only provide project directories. Single files are not allowed."
        case crash_test.error_codes.MULTIPASS_GENERIC_ERROR:
            return f"{Fore.RED}multipass: error: "
        case crash_test.error_codes.MULTIPASS_NOT_INSTALLED_ERROR:
            return "crashtest: error: Multipass is not installed!"
        case crash_test.error_codes.INVALID_INSTANCE_NAME_ERROR:
            return "crashtest: error: invalid instance name. The instance name should be like this: "
        case crash_test.error_codes.NO_DEPENDENCIES_FOUND_ERROR:
            return f"{Fore.YELLOW}No dependencies detected.{Style.RESET_ALL}\n"
        case crash_test.error_codes.NO_SUPPORTED_REQUIREMENTS_FILE_FOUND_ERROR:
            return f"{Fore.YELLOW}No supported project requirements file was found!{Style.RESET_ALL}\n"
